﻿/// Copyright © 2013 Microsoft Corporation. 
/// This project is released under the Microsoft Public License (MS-PL)
///
namespace DataMorph.Processing.Tests
{
    using System;
    using DataMorph.Common;
    using DataMorph.Common.DataAccess;
    using DataMorph.Common.Storage;
    using DataMorph.Generator;

    using Microsoft.WindowsAzure.Storage;
    using Microsoft.WindowsAzure.Storage.Auth;

    using Xunit;

    public class IntegrationTests
    {
        [Fact]
        public void CanProcess()
        {
            var now = DateTime.UtcNow;
            var dataSetName = "test-" + now.Ticks;
            var tagCount = 100;
            var dayCount = 2;
            var fileCount = 10;
            var startFrom = now - TimeSpan.FromDays(dayCount);
            //var storageAccount = CloudStorageAccount.DevelopmentStorageAccount;
            var storageAccount = new CloudStorageAccount(new StorageCredentials("dmitriy", "9H4Y8T22pd5F5GCBnOD6HE5aO44CfQle3IvxKmwHrb83oNEmgMKWkzy2rVPlALwPu53eIg2tb3Y5/Z+/v+2ITQ=="), true);
            var cloudStorageAccount = new CloudStorageAccountWrapper(storageAccount);

            this.GenerateData(cloudStorageAccount, dataSetName, tagCount, fileCount, dayCount, startFrom);

            this.GenerateWorkRequests(cloudStorageAccount, new GenerationParameters { Name = dataSetName, NumberOfFiles = fileCount, StorageAccountKey = "9H4Y8T22pd5F5GCBnOD6HE5aO44CfQle3IvxKmwHrb83oNEmgMKWkzy2rVPlALwPu53eIg2tb3Y5/Z+/v+2ITQ==", StorageAccountName = "dmitriy" });

            var dataAccess = new DataAccess(cloudStorageAccount);
            var workQueue = new WorkQueue(cloudStorageAccount, 32);

            var tagDataFileParser = new TagDataFileParser();

            var progressQueue = new ProgressQueue(cloudStorageAccount, 32);
            var progressNotifier = new ProgressNotifier(progressQueue);

            var workProcessor = new WorkProcessor(
                                            workQueue,
                                            dataAccess,
                                            tagDataFileParser,
                                            progressNotifier,
                                            "dep1",
                                            "inst1");

            workProcessor.Run();
        }

        private void GenerateData(ICloudStorageAccount cloudStorageAccount, string dataSetName, int tagCount, int fileCount, int dayCount, DateTime startFrom)
        {
            var tagGenerator = new TagGenerator();
            var tagDataGenerator = new TagDataGenerator();
            var tagDataFileGenerator = new TagDataFileGenerator(tagDataGenerator);

            var tagDataSetGenerator = new TagDataSetGenerator(tagDataFileGenerator, tagGenerator, (dataSet, day) => new BlobFileWriter(cloudStorageAccount, dataSet, day));
            tagDataSetGenerator.Generate(new GenerationParameters
            {
                Name = dataSetName,
                NumberOfTags = tagCount,
                NumberOfDays = dayCount,
                NumberOfFiles = fileCount,
                MinFileSize = 1 * 1000 * 1000,
                MaxFileSize = 4 * 1000 * 1000,
                StartDateTime = startFrom,
                Requested = DateTime.UtcNow
            });
        }

        private void GenerateWorkRequests(ICloudStorageAccount cloudStorageAccount, GenerationParameters parameters)
        {
            var workQueue = new WorkQueue(cloudStorageAccount, 32);
            var workScheduler = new WorkScheduler(workQueue, (accountName, key) => new CloudStorageAccountWrapper(accountName, key));
            //var workScheduler = new WorkScheduler(workQueue, (accountName, key) => new CloudStorageAccountWrapper(CloudStorageAccount.DevelopmentStorageAccount));

            workScheduler.ScheduleWork(parameters);
        }
    }
}